'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _implementsReflectionImpl = require('../implements/ReflectionImpl');

var _Action = require('./Action');

var _Store = require('./Store');

/**
 * Do not control application data in the domain, asynchronous processing also does not.
 * Application data that all keep in the Store.
 * Asynchronous processing should be implemented as external function.
 *
 * @class Domain
 */

var Domain = (function () {

  /**
   * @constructor
   */

  function Domain() {
    _classCallCheck(this, Domain);

    this.stores = new Map();
    this.actions = new Map();

    this.initialize();
  }

  _createClass(Domain, [{
    key: 'initialize',

    /**
     *
     */
    value: function initialize() {}
  }, {
    key: 'registerAction',

    /**
     * @param {Action} action
     */
    value: function registerAction(action) {
      if (!(action instanceof _Action['default'])) {
        throw new Error('Given instance of ' + action.getClassName() + ' is not Action');
      }

      var ActionClass = Object.getPrototypeOf(action).constructor;
      if (this.actions.has(ActionClass)) {
        throw new Error('' + action.getClassName() + ' already exists in this domain.');
      } else {
        this.actions.set(ActionClass, action);
      }
    }
  }, {
    key: 'getAction',

    /**
     * @param {Function} ActionClass
     * @returns {Action}
     */
    value: function getAction(ActionClass) {
      if (!this.actions.has(ActionClass)) {
        throw new Error('' + ActionClass.constructor.name + ' is not registered as Action.');
      }

      return this.actions.get(ActionClass);
    }
  }, {
    key: 'registerStore',

    /**
     * @param {Store} store
     */
    value: function registerStore(store) {
      if (!(store instanceof _Store['default'])) {
        throw new Error('Given instance of ' + store.getClassName() + ' is not Store');
      }

      var StoreClass = Object.getPrototypeOf(store).constructor;
      if (this.stores.has(StoreClass)) {
        throw new Error('' + store.getClassName() + ' already exists in this domain.');
      } else {
        this.stores.set(StoreClass, store);
      }
    }
  }, {
    key: 'getStore',

    /**
     * @param {Function} StoreClass
     * @returns {Store}
     */
    value: function getStore(StoreClass) {
      if (!this.stores.has(StoreClass)) {
        throw new Error('' + StoreClass.constructor.name + ' is not registered as Store.');
      }

      return this.stores.get(StoreClass);
    }
  }, {
    key: 'subscribeActionObservableFromStore',

    /**
     * All `Store` subscribe to all `Action`'s Observable
     */
    value: function subscribeActionObservableFromStore() {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = this.stores.values()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var store = _step.value;
          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = this.actions.values()[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var action = _step2.value;

              store.plugAction(action);
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2['return']) {
                _iterator2['return']();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator['return']) {
            _iterator['return']();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }, {
    key: 'mountRootComponent',

    /**
     * @param {Component} Component
     * @param {Element} mountNode
     * @returns {ReactComponent}
     */
    value: function mountRootComponent(React, Component, mountNode) {
      this.subscribeActionObservableFromStore();
      return React.render(React.createElement(Component, { domain: this }), mountNode);
    }
  }, {
    key: 'getObservables',

    /**
     * `getObservables` method not implemented anything initially.
     * This method will call from the `provideObservables`.
     * Sync the prop component the returned object as a template.
     * Good for most `observables` caching as a property value.
     *
     * @example
     * ```
     * observables = null;
     *
     * getObservables() {
     *   if (!this.observables) {
     *   this.observables = {
     *       update$ : this.getStore(AppStore).items$
     *     };
     *   }
     *
     *   return this.observables;
     * }
     * ```
     *
     * @returns {Object<string, Rx.Observable>}
     */
    value: function getObservables() {
      throw new Error(this.getClassName() + ':`getObservables` is abstract method.' + 'You should implements in sub-class');
    }
  }, {
    key: 'getClassName',

    /**
     * @returns {string}
     */
    value: function getClassName() {
      return _implementsReflectionImpl['default'].getClassName.apply(this);
    }
  }]);

  return Domain;
})();

exports['default'] = Domain;

/**
 * @type {Map<Function, Store>}
 */

/**
 * @type {Map<Function, Action>}
 */

// implements you want