'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _Subject = require('./Subject');

var _implementsReflectionImpl = require('../implements/ReflectionImpl');

var _implementsSubscriberImpl = require('../implements/SubscriberImpl');

/**
 * The role of this class is the Flux `Store`.
 * Subscribe the anonymous Observable provided the Action.
 * Itself has any Observable to expose Domain property.
 *
 * @example
 * ```
 * import {Store, Subject} from 'loxe';
 *
 * export default class AppStore extends Store {
 *   _items = [];
 *   items$ = Subject.property([]);
 *   initialize() {
 *     this.subscribe('ADD_ITEM', (item) => {
 *       this._items.unshift(item);
 *       this.items$.push(this._items);
 *     });
 *   }
 * }
 * ```
 *
 * @class Store
 */

var Store = (function () {

  /**
   * @constructor
   */

  function Store() {
    _classCallCheck(this, Store);

    this.plugStream$ = _Subject['default'].stream();

    this.initialize();
  }

  _createClass(Store, [{
    key: 'initialize',

    /**
     *
     */
    value: function initialize() {}
  }, {
    key: 'plugAction',

    /**
     *
     * @param {Action} action
     */
    value: function plugAction(action) {
      this.subscribe(action.eventStream$, this.plugStream$.push.bind(this.plugStream$));
    }
  }, {
    key: 'getEvent',

    /**
     *
     * @param {string} eventName
     * @returns {Rx.Observable<*>}
     */
    value: function getEvent(eventName) {
      return this.plugStream$.filter(function (_ref) {
        var event = _ref.event;
        return event === eventName;
      }).map(function (_ref2) {
        var payload = _ref2.payload;
        return payload;
      });
    }
  }, {
    key: 'subscribeEvent',

    /**
     * @param {string} eventName
     * @param {Function} observer
     */
    value: function subscribeEvent(eventName, observer) {
      this.subscribe(this.getEvent(eventName), observer);
    }
  }, {
    key: 'subscribe',

    /**
     * Delegate to `SubscriberImpl.subscribe()`
     * Subscribe to if the first argument is a string, it filtered as the event name.
     *
     * @param {string|Rx.Observable<ActionData>} observable$
     * @param {function} observer
     */
    value: function subscribe(observable$, observer) {
      if (typeof observable$ === 'string') {
        observable$ = this.getEvent(observable$);
      }
      _implementsSubscriberImpl['default'].subscribe.apply(this, [observable$, observer]);
    }
  }, {
    key: 'unsubscribeAll',

    /**
     * Delegate to `SubscriberImpl.unsubscribeAll()`
     */
    value: function unsubscribeAll() {
      _implementsSubscriberImpl['default'].unsubscribeAll.apply(this);
    }
  }, {
    key: 'getClassName',

    /**
     * @returns {string}
     */
    value: function getClassName() {
      return _implementsReflectionImpl['default'].getClassName.apply(this);
    }
  }]);

  return Store;
})();

exports['default'] = Store;

/**
 *
 * @type {Kefir.Stream<ActionData>}
 * @private
 */

// implements you want