'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _kefir = require('kefir');

var Kefir = _kefir;

/**
 * @class Subject
 */

var Subject = (function () {
  function Subject() {
    _classCallCheck(this, Subject);
  }

  _createClass(Subject, null, [{
    key: 'stream',

    /**
     * Subject for `Component` ui events & `Action` publish events.
     *
     * @returns {Kefir.Stream}
     */
    value: function stream() {
      return Subject.create(Kefir.Stream);
    }
  }, {
    key: 'property',

    /**
     * Subject for `Store` data property.
     * Even when starting to subscribe to retained latest value will be published.
     *
     * @param {*} initialValue
     * @returns {Kefir.Property}
     */
    value: function property(initialValue) {
      return Subject.create(Kefir.Property, initialValue);
    }
  }, {
    key: 'create',

    /**
     * @param {Kefir.Observable} BaseClass
     * @param {*} initialValue
     * @returns {Kefir.Observable}
     */
    value: function create(BaseClass, initialValue) {

      function _subject() {
        _subject._emitValue.apply(_subject, arguments);
      }

      for (var key in BaseClass.prototype) {
        _subject[key] = BaseClass.prototype[key];
      }

      _subject.push = _subject._emitValue;
      _subject.error = _subject._emitError;
      _subject.end = _subject._emitEnd;

      BaseClass.call(_subject);

      if (initialValue !== undefined) {
        _subject._active = true;
        _subject._currentEvent = { type: 'value', value: initialValue, current: true };
      }

      return _subject;
    }
  }]);

  return Subject;
})();

exports['default'] = Subject;