'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) subClass.__proto__ = superClass; }

var _kefirCombinetemplate = require('kefir.combinetemplate');

var kefirCombineTemplate = _kefirCombinetemplate;

var _objectAssign = require('object-assign');

var assign = _objectAssign;

var _implementsSubscriberImpl = require('../implements/SubscriberImpl');

var _implementsReflectionImpl = require('../implements/ReflectionImpl');

var _utilsDecoratable = require('../utils/decoratable');

/**
 * @param {Component} Component
 * @param {React} React
 * @param {Function} receiveObservablesHandler
 * @returns {ActionsProvider}
 */
function provideObservables(Component, React, receiveObservablesHandler) {

  /**
   * @class ObservablesProvider
   */

  var ObservablesProvider = (function (_React$Component) {
    function ObservablesProvider() {
      _classCallCheck(this, ObservablesProvider);

      if (_React$Component != null) {
        _React$Component.apply(this, arguments);
      }
    }

    _inherits(ObservablesProvider, _React$Component);

    _createClass(ObservablesProvider, [{
      key: 'subscribe',

      /**
       * Delegate to `SubscriberImpl.subscribe()`
       *
       * @param {Observable} observable$
       * @param {function} observer
       */
      value: function subscribe(observable$, observer) {
        _implementsSubscriberImpl['default'].subscribe.apply(this, [observable$, observer]);
      }
    }, {
      key: 'unsubscribeAll',

      /**
       * Delegate to `SubscriberImpl.unsubscribeAll()`
       */
      value: function unsubscribeAll() {
        _implementsSubscriberImpl['default'].unsubscribeAll.apply(this);
      }
    }, {
      key: 'getDisplayName',

      /**
       * for react dev-tools
       * @returns {string}
       */
      value: function getDisplayName() {
        return _implementsReflectionImpl['default'].getClassName.apply(this);
      }
    }, {
      key: 'componentWillMount',

      /**
       *
       */
      value: function componentWillMount() {
        if (!this.context.getObservables) {
          throw new Error('The context does not have `getObservables`.' + 'Make sure the ancestral component provides the domain context, use `@provideContext`.');
        }

        var observables = this.context.getObservables();
        var stateObject = receiveObservablesHandler(observables);

        this.subscribe(kefirCombineTemplate(stateObject), this.setState.bind(this));
      }
    }, {
      key: 'componentWillUnmount',

      /**
       * To notify that the component unmounted to the domain.
       */
      value: function componentWillUnmount() {
        this.unsubscribeAll();
      }
    }, {
      key: 'render',

      /**
       * Values that from Observables are saved in `state`.
       * Delegate to components within props.
       *
       * @returns {*}
       */
      value: function render() {
        return React.createElement(Component, assign({}, this.props, this.state));
      }
    }], [{
      key: '_originalComponent',

      /**
       * @type {Component}
       * @private
       */
      value: Component,
      enumerable: true
    }, {
      key: 'contextTypes',

      /**
       * @type {Object<string, function>}
       */
      value: {
        getObservables: React.PropTypes.func.isRequired
      },
      enumerable: true
    }]);

    return ObservablesProvider;
  })(React.Component);

  return ObservablesProvider;
}

exports['default'] = (0, _utilsDecoratable['default'])(provideObservables);