'use strict';

exports['default'] = decoratable;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

/**
 * can be used as decorators
 * Main purpose the higher-order component to use as a Decorator.
 *
 * https://github.com/wycats/javascript-decorators
 *
 * ```
 *   let decoratableSubject = decoratable(subject);
 *   @decoratableSubject(foo, bar, baz)
 *   class AcmeClass {
 *     // acme implements
 *   }
 *
 *   // same as below
 *
 *   class AcmeClass {
 *     // acme implements
 *   }
 *   subject(AcmeClass, foo, bar, baz);
 * ```
 *
 * @param {Function} subjectFunc
 */

function decoratable(subjectFunc) {
  return function () {
    for (var _len = arguments.length, initialArgs = Array(_len), _key = 0; _key < _len; _key++) {
      initialArgs[_key] = arguments[_key];
    }

    // Decorator
    if (initialArgs.length < subjectFunc.length) {
      var _ret = (function () {
        var diff = subjectFunc.length - initialArgs.length;

        return {
          v: function decorateWrapper() {
            for (var _len2 = arguments.length, targetKeyDescriptor = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
              targetKeyDescriptor[_key2] = arguments[_key2];
            }

            return subjectFunc.apply(null, [].concat(_toConsumableArray(targetKeyDescriptor.slice(0, diff)), initialArgs));
          }
        };
      })();

      if (typeof _ret === 'object') return _ret.v;
    }

    // Normally
    return subjectFunc.apply(null, initialArgs);
  };
}